==> cuteradio_0.0.2.tar.gz: cuteradio-0.0.2/src/maemo5/nowplayingbutton.cpp <==
/*
 * Copyright (C) 2014 Stuart Howarth <showarth@marxoft.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 3, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <QPaintEvent>
#include <QTimer>
#include <QPainter>
#include <QApplication>
#include <QDesktopWidget>
#include "nowplayingbutton.h"
#include "nowplayingwindow.h"
#include "windowstack.h"
#include "screen.h"
#include "../base/station.h"
#include "../mediaplayer/phonon/mediaplayer.h"

NowPlayingButton* NowPlayingButton::self = 0;

NowPlayingButton::NowPlayingButton(StackedWindow *parent) :
    QWidget(parent),
    m_animationTimer(new QTimer(this)),
    m_currentFrame(0),
    m_ready(false),
    m_moved(false)
{
    if (!self) {
        self = this;
    }

    this->setFixedSize(112, 70);
    this->setAttribute(Qt::WA_OpaquePaintEvent);
    this->setAttribute(Qt::WA_Maemo5NonComposited);
    this->hide();

    m_animationTimer->setInterval(100);
    m_images.reserve(13);
    m_images.append(QPixmap("/usr/share/icons/hicolor/scalable/hildon/mediaplayer_nowplaying_indicator_pause.png"));

    for (int i = 1; i <= 12; i++) {
        m_images.append(QPixmap(QString("/usr/share/icons/hicolor/scalable/hildon/mediaplayer_nowplaying_indicator%1.png").arg(i)));
    }

    this->connect(m_animationTimer, SIGNAL(timeout()), this, SLOT(nextFrame()));
    this->connect(MediaPlayer::instance(), SIGNAL(currentStationChanged(Station*)), this, SLOT(onCurrentStationChanged(Station*)));
    this->connect(MediaPlayer::instance(), SIGNAL(stateChanged(Phonon::State)), this, SLOT(onPlaybackStateChanged()));
    this->connect(WindowStack::instance(), SIGNAL(currentWindowChanged(StackedWindow*)), this, SLOT(onCurrentWindowChanged(StackedWindow*)));
    this->connect(QApplication::desktop(), SIGNAL(resized(int)), this, SLOT(onScreenOrientationChanged()));
    this->connect(Screen::instance(), SIGNAL(screenLockStateChanged(bool)), this, SLOT(onScreenLockStateChanged(bool)));
}

NowPlayingButton::~NowPlayingButton() {}

NowPlayingButton* NowPlayingButton::instance() {
    return !self ? new NowPlayingButton : self;
}

void NowPlayingButton::showEvent(QShowEvent *event) {
    if (!this->moved()) {
        QRect screen = QApplication::desktop()->screenGeometry();
        this->move(screen.width() - (this->width() + 8), screen.height() - (this->height() + 56));
        this->setMoved(true);
    }

    this->triggerAnimation();
    event->accept();
}

void NowPlayingButton::hideEvent(QHideEvent *event) {
    this->stopAnimation();
    event->accept();
}

bool NowPlayingButton::ready() const {
    return m_ready;
}

void NowPlayingButton::setReady(bool ready) {
    if (ready != this->ready()) {
        m_ready = ready;
        emit readyChanged(ready);
    }
}

bool NowPlayingButton::moved() const {
    return m_moved;
}

void NowPlayingButton::setMoved(bool moved) {
    m_moved = moved;
}

void NowPlayingButton::poke() {
    if (this->ready()) {
        this->show();
        this->raise();
    }
}

void NowPlayingButton::paintEvent(QPaintEvent *event) {
    QPainter painter(this);
    painter.drawPixmap(0, 0, m_images.at(m_currentFrame));
    event->accept();
}

void NowPlayingButton::mousePressEvent(QMouseEvent *event) {
    event->accept();
}

void NowPlayingButton::mouseReleaseEvent(QMouseEvent *event) {
    if (this->rect().contains(event->pos())) {
        this->showNowPlayingWindow();
    }

    event->accept();
}

void NowPlayingButton::onCurrentStationChanged(Station *station) {
    this->setReady(station ? true : false);

    if ((this->ready()) && (WindowStack::instance()->currentWindow()->hasNowPlayingButton())) {
        this->poke();
    }
    else {
        this->hide();
    }
}

void NowPlayingButton::onPlaybackStateChanged() {
    if (MediaPlayer::instance()->playing()) {
        this->triggerAnimation();
    }
    else {
        this->stopAnimation();
    }
}

void NowPlayingButton::onCurrentWindowChanged(StackedWindow *window) {
    if ((window) && (window->hasNowPlayingButton())) {
        this->setParent(window);
        this->setMoved(false);
        this->poke();
    }
    else {
        this->hide();
    }
}

void NowPlayingButton::onScreenLockStateChanged(bool locked) {
    if (locked) {
        this->stopAnimation();
    }
    else {
        this->triggerAnimation();
    }
}

void NowPlayingButton::onScreenOrientationChanged() {
    QRect screen = QApplication::desktop()->screenGeometry();
    this->move(screen.width() - (this->width() + 8), screen.height() - (this->height() + 56));
    this->setMoved(true);
}

void NowPlayingButton::nextFrame() {
    switch (m_currentFrame) {
    case 12:
        m_currentFrame = 1;
        break;
    default:
        m_currentFrame++;
        break;
    }

    this->update();
}

void NowPlayingButton::stopAnimation() {
    m_animationTimer->stop();
    m_currentFrame = 0;
    this->update();
}

void NowPlayingButton::triggerAnimation() {
    if ((this->isVisible()) && (MediaPlayer::instance()->state() == Phonon::PlayingState) && (!m_animationTimer->isActive())) {
        m_animationTimer->start();
    }
}

void NowPlayingButton::showNowPlayingWindow() {
    NowPlayingWindow *window = new NowPlayingWindow(WindowStack::instance()->currentWindow());
    window->show();
}

==> cuteradio_0.1.2.tar.gz: cuteradio-0.1.2/src/maemo5/nowplayingbutton.cpp <==
/*
 * Copyright (C) 2014 Stuart Howarth <showarth@marxoft.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 3, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <QPaintEvent>
#include <QTimer>
#include <QPainter>
#include <QApplication>
#include <QDesktopWidget>
#include "nowplayingbutton.h"
#include "nowplayingwindow.h"
#include "windowstack.h"
#include "screen.h"
#include "../base/station.h"
#include "../mediaplayer/phonon/mediaplayer.h"

NowPlayingButton* NowPlayingButton::self = 0;

NowPlayingButton::NowPlayingButton(StackedWindow *parent) :
    QWidget(parent),
    m_animationTimer(new QTimer(this)),
    m_currentFrame(0),
    m_ready(false),
    m_moved(false)
{
    if (!self) {
        self = this;
    }

    this->setFixedSize(112, 70);
    this->setAttribute(Qt::WA_OpaquePaintEvent);
    this->setAttribute(Qt::WA_Maemo5NonComposited);
    this->hide();

    m_animationTimer->setInterval(100);
    m_images.reserve(13);
    m_images.append(QPixmap("/usr/share/icons/hicolor/scalable/hildon/mediaplayer_nowplaying_indicator_pause.png"));

    for (int i = 1; i <= 12; i++) {
        m_images.append(QPixmap(QString("/usr/share/icons/hicolor/scalable/hildon/mediaplayer_nowplaying_indicator%1.png").arg(i)));
    }

    this->connect(m_animationTimer, SIGNAL(timeout()), this, SLOT(nextFrame()));
    this->connect(MediaPlayer::instance(), SIGNAL(currentStationChanged(Station*)), this, SLOT(onCurrentStationChanged(Station*)));
    this->connect(MediaPlayer::instance(), SIGNAL(stateChanged(MediaState::State)), this, SLOT(onPlaybackStateChanged()));
    this->connect(WindowStack::instance(), SIGNAL(currentWindowChanged(StackedWindow*)), this, SLOT(onCurrentWindowChanged(StackedWindow*)));
    this->connect(QApplication::desktop(), SIGNAL(resized(int)), this, SLOT(onScreenOrientationChanged()));
    this->connect(Screen::instance(), SIGNAL(screenLockStateChanged(bool)), this, SLOT(onScreenLockStateChanged(bool)));
}

NowPlayingButton::~NowPlayingButton() {}

NowPlayingButton* NowPlayingButton::instance() {
    return !self ? new NowPlayingButton : self;
}

void NowPlayingButton::showEvent(QShowEvent *event) {
    if (!this->moved()) {
        QRect screen = QApplication::desktop()->screenGeometry();
        this->move(screen.width() - (this->width() + 8), screen.height() - (this->height() + 56));
        this->setMoved(true);
    }

    this->triggerAnimation();
    event->accept();
}

void NowPlayingButton::hideEvent(QHideEvent *event) {
    this->stopAnimation();
    event->accept();
}

bool NowPlayingButton::ready() const {
    return m_ready;
}

void NowPlayingButton::setReady(bool ready) {
    if (ready != this->ready()) {
        m_ready = ready;
        emit readyChanged(ready);
    }
}

bool NowPlayingButton::moved() const {
    return m_moved;
}

void NowPlayingButton::setMoved(bool moved) {
    m_moved = moved;
}

void NowPlayingButton::poke() {
    if (this->ready()) {
        this->show();
        this->raise();
    }
}

void NowPlayingButton::paintEvent(QPaintEvent *event) {
    QPainter painter(this);
    painter.drawPixmap(0, 0, m_images.at(m_currentFrame));
    event->accept();
}

void NowPlayingButton::mousePressEvent(QMouseEvent *event) {
    event->accept();
}

void NowPlayingButton::mouseReleaseEvent(QMouseEvent *event) {
    if (this->rect().contains(event->pos())) {
        this->showNowPlayingWindow();
    }

    event->accept();
}

void NowPlayingButton::onCurrentStationChanged(Station *station) {
    this->setReady(station ? true : false);

    if ((this->ready()) && (WindowStack::instance()->currentWindow()->hasNowPlayingButton())) {
        this->poke();
    }
    else {
        this->hide();
    }
}

void NowPlayingButton::onPlaybackStateChanged() {
    if (MediaPlayer::instance()->playing()) {
        this->triggerAnimation();
    }
    else {
        this->stopAnimation();
    }
}

void NowPlayingButton::onCurrentWindowChanged(StackedWindow *window) {
    if ((window) && (window->hasNowPlayingButton())) {
        this->setParent(window);
        this->setMoved(false);
        this->poke();
    }
    else {
        this->hide();
    }
}

void NowPlayingButton::onScreenLockStateChanged(bool locked) {
    if (locked) {
        this->stopAnimation();
    }
    else {
        this->triggerAnimation();
    }
}

void NowPlayingButton::onScreenOrientationChanged() {
    QRect screen = QApplication::desktop()->screenGeometry();
    this->move(screen.width() - (this->width() + 8), screen.height() - (this->height() + 56));
    this->setMoved(true);
}

void NowPlayingButton::nextFrame() {
    switch (m_currentFrame) {
    case 12:
        m_currentFrame = 1;
        break;
    default:
        m_currentFrame++;
        break;
    }

    this->update();
}

void NowPlayingButton::stopAnimation() {
    m_animationTimer->stop();
    m_currentFrame = 0;
    this->update();
}

void NowPlayingButton::triggerAnimation() {
    if ((this->isVisible()) && (MediaPlayer::instance()->playing()) && (!m_animationTimer->isActive())) {
        m_animationTimer->start();
    }
}

void NowPlayingButton::showNowPlayingWindow() {
    NowPlayingWindow *window = new NowPlayingWindow(WindowStack::instance()->currentWindow());
    window->show();
}
